@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2016 Enrico M. Crisostomo
+ * Copyright (c) 2014-2021 Enrico M. Crisostomo
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -15,7 +15,7 @@
  */
 /**
  * @file
- * @brief OS X FSEvents monitor.
+ * @brief macOS FSEvents monitor.
  *
  * @copyright Copyright (c) 2014-2016 Enrico M. Crisostomo
  * @license GNU General Public License v. 3.0
@@ -28,31 +28,47 @@
 
 #  include "monitor.hpp"
 #  include <CoreServices/CoreServices.h>
-#  ifdef HAVE_CXX_MUTEX
-#    include <mutex>
-#  endif
+
 namespace fsw
 {
   /**
-   * @brief OS X FSEvents monitor.
+   * @brief macOS FSEvents monitor.
    *
-   * This monitor is built upon the _FSEvents_ API of the Apple OS X kernel.
+   * This monitor is built upon the _FSEvents_ API of the Apple macOS kernel.
    */
   class fsevents_monitor : public monitor
   {
-    REGISTER_MONITOR(fsevents_monitor, fsevents_monitor_type);
-
   public:
+
+    /**
+     * @brief Custom monitor property used to enable the kFSEventStreamCreateFlagNoDefer flag in the event stream.
+     *
+     * If you specify this flag and more than latency seconds have elapsed since
+     * the last event, your app will receive the event immediately.  The
+     * delivery of the event resets the latency timer and any further events
+     * will be delivered after latency seconds have elapsed.  This flag is
+     * useful for apps that are interactive and want to react immediately to
+     * changes but avoid getting swamped by notifications when changes are
+     * occurring in rapid succession.  If you do not specify this flag, then
+     * when an event occurs after a period of no events, the latency timer is
+     * started. Any events that occur during the next latency seconds will be
+     * delivered as one group (including that first event).  The delivery of the
+     * group of events resets the latency timer and any further events will be
+     * delivered after latency seconds.  This is the default behavior and is
+     * more appropriate for background, daemon or batch processing apps.
+     *
+     * @sa https://developer.apple.com/documentation/coreservices/kfseventstreamcreateflagnodefer
+     */
+    static constexpr const char *DARWIN_EVENTSTREAM_NO_DEFER = "darwin.eventStream.noDefer";
+
     /**
      * @brief Constructs an instance of this class.
      */
     fsevents_monitor(std::vector<std::string> paths,
                      FSW_EVENT_CALLBACK *callback,
                      void *context = nullptr);
-    /**
-     * @brief Destroys an instance of this class.
-     */
-    virtual ~fsevents_monitor();
+    fsevents_monitor(const fsevents_monitor& orig) = delete;
+    fsevents_monitor& operator=(const fsevents_monitor& that) = delete;
 
   protected:
     /**
@@ -70,9 +86,6 @@ namespace fsw
     void on_stop() override;
 
   private:
-    fsevents_monitor(const fsevents_monitor& orig) = delete;
-    fsevents_monitor& operator=(const fsevents_monitor& that) = delete;
-
     static void fsevents_callback(ConstFSEventStreamRef streamRef,
                                   void *clientCallBackInfo,
                                   size_t numEvents,
@@ -80,11 +93,10 @@ namespace fsw
                                   const FSEventStreamEventFlags eventFlags[],
                                   const FSEventStreamEventId eventIds[]);
 
-#  ifdef HAVE_CXX_MUTEX
-    std::mutex run_mutex;
-#  endif
     FSEventStreamRef stream = nullptr;
     CFRunLoopRef run_loop = nullptr;
+    bool no_defer();
+    void create_stream(CFArrayRef pathsToWatch);
   };
 }
 
