@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2016 Enrico M. Crisostomo
+ * Copyright (c) 2014-2021 Enrico M. Crisostomo
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -29,9 +29,9 @@
  * `fswatch` is a cross-platform file change monitor currently supporting the
  * following backends:
  *
- *   - A monitor based on the _FSEvents_ API of Apple OS X.
+ *   - A monitor based on the _FSEvents_ API of Apple macOS.
  *   - A monitor based on _kqueue_, an event notification interface introduced
- *     in FreeBSD 4.1 and supported on most *BSD systems (including OS X).
+ *     in FreeBSD 4.1 and supported on most *BSD systems (including macOS).
  *   - A monitor based on _File Events Notification_, an event notification API
  *     of the Solaris/Illumos kernel.
  *   - A monitor based on _inotify_, a Linux kernel subsystem that reports
@@ -275,6 +275,18 @@
 /**
  * @page history History
  *
+ * @section v1110 11:1:0
+ *
+ *   - Fix monitor_factory::create_monitor ignoring the monitor type and always
+ *     returning the default system monitor. (Issue 218: fswatch v1.13 ignores
+ *     the --monitor parameter and always uses the default monitor).
+ *
+ * @section v1100 11:0:0
+ *
+ *   - Refactor the monitor_factory class so that available monitor types are
+ *     determined at compile time.  (Issue 142: Static library will not have any
+ *     monitor type available).
+ *
  * @section v1011 10:1:1
  *
  *   - Migrate usages of POSIX regular expressions (<regex.h>) to the C++11
@@ -786,7 +798,7 @@ FSW_STATUS fsw_destroy_session(const FSW_HANDLE handle)
 
       void *context = session->monitor->get_context();
 
-      if (!context)
+      if (context)
       {
         session->monitor->set_context(nullptr);
         delete static_cast<fsw_callback_context *> (context);
